"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.KmsProvider = void 0;
const web3_provider_engine_1 = __importDefault(require("web3-provider-engine"));
const hooked_wallet_1 = __importDefault(require("web3-provider-engine/subproviders/hooked-wallet"));
const rpc_1 = __importDefault(require("web3-provider-engine/subproviders/rpc"));
const ethereumjs_tx_1 = require("ethereumjs-tx");
const ethereumjs_common_1 = __importDefault(require("ethereumjs-common"));
const ethereumjs_util_1 = require("ethereumjs-util");
const aws_kms_signer_1 = require("aws-kms-signer");
const ethereum_1 = require("./ethereum");
class KmsProvider {
    constructor(endpoint, kmsOptions, networkOrNetworkOptions) {
        this.cacheAccounts = [];
        this.engine = new web3_provider_engine_1.default();
        this.signers = kmsOptions.keyIds.map((keyId) => new aws_kms_signer_1.KmsSigner(keyId, {
            credentials: kmsOptions.credential,
            region: kmsOptions.region,
        }));
        this.networkOrNetworkOptions = networkOrNetworkOptions;
        this.ethereum = new ethereum_1.Ethereum(endpoint);
        this.engine.addProvider(new hooked_wallet_1.default({
            getAccounts: (callback) => {
                this.getAccounts()
                    .then((accounts) => {
                    callback(null, accounts);
                })
                    .catch((err) => {
                    callback(err);
                });
            },
            signTransaction: (txData, callback) => {
                this.signTransaction(txData)
                    .then((rawTx) => {
                    callback(null, rawTx);
                })
                    .catch((err) => callback(err));
            },
            signMessage: (msgParams, callback) => {
                this.signMessage(msgParams)
                    .then((signature) => {
                    callback(null, signature);
                })
                    .catch((err) => callback(err));
            },
        }));
        this.engine.addProvider(new rpc_1.default({ rpcUrl: endpoint }));
        this.engine.start();
    }
    async getAccounts() {
        if (this.cacheAccounts.length !== 0)
            return this.cacheAccounts;
        const addresses = await Promise.all(this.signers.map((signer) => signer.getAddress()));
        this.cacheAccounts = addresses.map((address) => `0x${address.toString()}`);
        return this.cacheAccounts;
    }
    async signTransaction(txData) {
        const from = txData.from;
        const signer = this.resolveSigner(from);
        if (!signer) {
            throw new Error(`Account not found: ${from}`);
        }
        const tx = await this.createTransaction(txData);
        const digest = tx.hash(false);
        const signature = await signer.sign(digest);
        // TODO: move to signature.ts
        const vStr = (signature.v + tx.getChainId() * 2 + 8).toString(16);
        const length = vStr.length + (vStr.length % 2);
        const v = Buffer.from(vStr.padStart(length, "0"), "hex");
        tx.r = signature.r;
        tx.s = signature.s;
        tx.v = v;
        const rawTx = `0x${tx.serialize().toString("hex")}`;
        return rawTx;
    }
    async signMessage(msgParams) {
        const from = msgParams.from;
        const signer = this.resolveSigner(from);
        if (!signer) {
            throw new Error(`Account not found: ${from}`);
        }
        const data = (0, ethereumjs_util_1.toBuffer)(msgParams.data);
        const digest = (0, ethereumjs_util_1.hashPersonalMessage)(data);
        const signature = await signer.sign(digest);
        return `0x${signature.toString()}`;
    }
    send(payload, callback) {
        this.engine.sendAsync(payload, callback);
    }
    sendAsync(payload, callback) {
        this.engine.sendAsync(payload, callback);
    }
    resolveSigner(address) {
        const index = this.cacheAccounts
            .map((account) => account.toLowerCase())
            .indexOf(address.toLowerCase());
        if (index !== -1) {
            return this.signers[index];
        }
    }
    async getTransactionOptions() {
        const isNetworkOptions = (x) => {
            return (typeof x === "object" &&
                typeof x.chainName === "string" &&
                typeof x.chainId === "number" &&
                typeof x.networkId === "number");
        };
        if (!this.networkOrNetworkOptions) {
            const networkId = Number.parseInt(await this.ethereum.netVersion());
            const common = ethereumjs_common_1.default.forCustomChain("mainnet", {
                chainId: networkId,
                networkId,
            }, "petersburg");
            return {
                common,
            };
        }
        if (isNetworkOptions(this.networkOrNetworkOptions)) {
            const networkOptions = this.networkOrNetworkOptions;
            return {
                common: ethereumjs_common_1.default.forCustomChain("mainnet", networkOptions),
            };
        }
        const newtwork = this.networkOrNetworkOptions;
        return { chain: newtwork };
    }
    async createTransaction(txData) {
        const opts = await this.getTransactionOptions();
        return new ethereumjs_tx_1.Transaction(txData, opts);
    }
}
exports.KmsProvider = KmsProvider;
